/*
 * File:
 *      main.c
 *
 * Description:
 * 	The testbed for dcihid.c
 *
 * History:
 *      2009/04/24: initially created
 *              David Kao
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <sys/types.h>

#include "dcihid.h"

/*
 * According to the kernel Documentation/usb/hiddev.txt, there are totally
 * 16 hiddevs as follows.
 * The hiddev interface is a char interface using the normal USB major,
 * with the minor numbers starting at 96 and finishing at 111.
 * /dev/hiddev0 c 180 96
 * /dev/hiddev1 c 180 97
 * /dev/hiddev2 c 180 98
 * /dev/hiddev3 c 180 99
 * /dev/hiddev4 c 180 100
 * /dev/hiddev5 c 180 101
 * /dev/hiddev6 c 180 102
 * /dev/hiddev7 c 180 103
 * /dev/hiddev8 c 180 104
 * /dev/hiddev9 c 180 105
 * /dev/hiddev10 c 180 106
 * /dev/hiddev11 c 180 107
 * /dev/hiddev12 c 180 108
 * /dev/hiddev13 c 180 109
 * /dev/hiddev14 c 180 110
 * /dev/hiddev15 c 180 111
 *
 * This testbed is just for the simple situatin that there is 
 * only one DCI 16-Port Photo/Relay HID device plugged. 
 * This device is opened with the pair of values,
 * (CardId, CardNo) = (3.0) by enumerating from 
 * /dev/hiddev0 to /dev/hiddev15
 *
 * If the device can be opened with dcihid_open(), a simple
 * read and write process will be demo. The device is configured as
 * two loopback connections as follows.
 * 1. Port 2 (Rely Out) --> Port 0 (Photo In)
 * 2. Port 3 (Rely Out) --> Port 1 (Photo In)
 */
int 
main(void)
{
#define DCIHID_16PR_CARDID	3
#define DCIHID_16PR_CARDNO	0

#define LINUX_HIDDEV_MAX	16
	static char *linux_hiddev[LINUX_HIDDEV_MAX] = {
 		"/dev/hiddev0", 
 		"/dev/hiddev1", 
 		"/dev/hiddev2", 
 		"/dev/hiddev3", 
 		"/dev/hiddev4", 
 		"/dev/hiddev5", 
 		"/dev/hiddev6", 
 		"/dev/hiddev7", 
 		"/dev/hiddev8", 
 		"/dev/hiddev9", 
 		"/dev/hiddev10", 
 		"/dev/hiddev11", 
 		"/dev/hiddev12", 
 		"/dev/hiddev13", 
 		"/dev/hiddev14", 
 		"/dev/hiddev15", 
	};

	int	  i;
	u_int32_t dcihid_handle = 0;

	for (i = 0; i < LINUX_HIDDEV_MAX; i++) {
  		dcihid_handle = dcihid_open(linux_hiddev[i], DCIHID_16PR_CARDID, DCIHID_16PR_CARDNO);
  		if (dcihid_handle != 0) break;
	}
	if (dcihid_handle == 0) {
		fprintf(stderr, "There is no DCI HID 16 Photo/Rely Device (CardID, CardNO) = (%d, %d) plugged\n", DCIHID_16PR_CARDID, DCIHID_16PR_CARDNO);

		return 1;
	}

	u_int32_t out_port, in_port;
	u_int8_t data;

	/*
	 * light off all of LEDs
	 */
	dcihid_write(dcihid_handle, 2, 0);
	dcihid_write(dcihid_handle, 3, 0);

	/*
	 * 1. Port 2 (Rely Out) --> Port 0 (Photo In)
	 */
	out_port = 2;
	in_port = 0;
	for (i = 0; i < 8; i++) {
		dcihid_write(dcihid_handle, out_port, 0x01 << i);
		dcihid_read(dcihid_handle, in_port, &data);
		printf("OutPort[%d] =  0x%x --> InPort[%d] = 0x%x\n", out_port, 0x01 << i, in_port, data);
		getchar();
	}
	dcihid_write(dcihid_handle, out_port, 0);

	/* 
 	 * 2. Port 3 (Rely Out) --> Port 1 (Photo In)
	 */
	out_port = 3;
	in_port = 1;
	for (i = 0; i < 8; i++) {
		dcihid_write(dcihid_handle, out_port, 0x01 << i);
		dcihid_read(dcihid_handle, in_port, &data);
		printf("OutPort[%d] =  0x%x --> InPort[%d] = 0x%x\n", out_port, 0x01 << i, in_port, data);
		getchar();
	}
	dcihid_write(dcihid_handle, out_port, 0);

	/*
	 * close
	 */
	dcihid_close(dcihid_handle);

  	return 0;
} /* main() */
